﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Encodings.ParameterConfigurationEncoding;
using HeuristicLab.Operators;
using HeuristicLab.Optimization;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.MetaOptimization {
  /// <summary>
  /// TODO
  /// </summary>
  [Item("AlgorithmEvaluator", "TODO")]
  [StorableClass]
  public class AlgorithmEvaluator : SingleSuccessorOperator {
    #region Parameter properties
    public ILookupParameter<IRandom> RandomParameter {
      get { return (LookupParameter<IRandom>)Parameters["Random"]; }
    }
    public ILookupParameter<IAlgorithm> AlgorithmParameter {
      get { return (LookupParameter<IAlgorithm>)Parameters["Algorithm"]; }
    }
    public ILookupParameter<IntValue> ProblemIndexParameter {
      get { return (LookupParameter<IntValue>)Parameters["ProblemIndex"]; }
    }
    public ILookupParameter<IntValue> RepetitionIndexParameter {
      get { return (LookupParameter<IntValue>)Parameters["RepetitionIndex"]; }
    }
    public ILookupParameter<ParameterConfigurationTree> ParameterConfigurationParameter {
      get { return (ILookupParameter<ParameterConfigurationTree>)Parameters["ParameterConfigurationTree"]; }
    }
    public LookupParameter<ResultCollection> ResultsParameter {
      get { return (LookupParameter<ResultCollection>)Parameters["Results"]; }
    }
    #endregion

    #region Constructors and Cloning
    public AlgorithmEvaluator()
      : base() {
      Parameters.Add(new LookupParameter<IRandom>("Random", "The pseudo random number generator which should be used to initialize the new random permutation."));
      Parameters.Add(new LookupParameter<IAlgorithm>("Algorithm", ""));
      Parameters.Add(new LookupParameter<IntValue>("ProblemIndex", ""));
      Parameters.Add(new LookupParameter<IntValue>("RepetitionIndex", ""));
      Parameters.Add(new LookupParameter<ParameterConfigurationTree>("ParameterConfigurationTree", ""));
      Parameters.Add(new LookupParameter<ResultCollection>("Results", ""));
    }

    [StorableConstructor]
    protected AlgorithmEvaluator(bool deserializing) : base(deserializing) { }
    protected AlgorithmEvaluator(AlgorithmEvaluator original, Cloner cloner) : base(original, cloner) { }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new AlgorithmEvaluator(this, cloner);
    }
    #endregion

    public override IOperation Apply() {
      bool isValidScope = RepetitionIndexParameter.ActualValue != null; // this is a workaround for OSGA, where the PMOEvaluator is executed on scopes which happen to contain additional subscopes which collide with the subscopes created for the repetitions/problems in PMOEvaluator
      if (!isValidScope) return base.Apply();

      ItemDictionary<StringValue, RunCollection> solutionCache =
        ResultsParameter.ActualValue.ContainsKey("SolutionCache") ?
        (ItemDictionary<StringValue, RunCollection>)ResultsParameter.ActualValue["SolutionCache"].Value :
        null;
      ParameterConfigurationTree parameterConfiguration = ParameterConfigurationParameter.ActualValue;
      IAlgorithm algorithm = AlgorithmParameter.ActualValue;
      int repetitionIndex = RepetitionIndexParameter.ActualValue.Value;

      if (solutionCache != null && solutionCache.Count(x => x.Key.Value == parameterConfiguration.ParameterInfoString) > 0) {
        algorithm.Runs.Add(solutionCache.Single(x => x.Key.Value == parameterConfiguration.ParameterInfoString).Value.ElementAt(repetitionIndex));
      } else {
        algorithm.StartSync(CancellationToken);
      }
      return base.Apply();
    }
  }
}
